/*
 * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug      8214126
 * @summary  Method signatures not formatted correctly in browser
 * @library  ../../lib/
 * @modules jdk.javadoc/jdk.javadoc.internal.tool
 * @build javadoc.tester.*
 * @run main TestMethodSignature
 */

import javadoc.tester.JavadocTester;

public class TestMethodSignature extends JavadocTester {

    public static void main(String... args) throws Exception {
        TestMethodSignature tester = new TestMethodSignature();
        tester.runTests();
    }

    @Test
    public void test() {
        javadoc("-d", "out",
                "-sourcepath", testSrc,
                "pkg");
        checkExit(Exit.OK);

        checkOutput("pkg/C.html", true,
                "<ul class=\"blockList\">\n" +
                        "<li class=\"blockList\">\n" +
                        "<h3>simpleMethod</h3>\n" +
                        "<pre class=\"methodSignature\">public static&nbsp;void&nbsp;simpleMethod&#8203;(int&nbsp;i,\n" +
                        "                                java.lang.String&nbsp;s,\n" +
                        "                                boolean&nbsp;b)</pre>\n" +
                        "<div class=\"block\">Simple method.</div>\n" +
                        "<dl>\n" +
                        "<dt><span class=\"paramLabel\">Parameters:</span></dt>\n" +
                        "<dd><code>i</code> - param 1</dd>\n" +
                        "<dd><code>s</code> - param 2</dd>\n" +
                        "<dd><code>b</code> - param 3</dd>\n" +
                        "</dl>\n" +
                        "</li>\n" +
                        "</ul>",
                "<ul class=\"blockList\">\n" +
                        "<li class=\"blockList\">\n" +
                        "<h3>annotatedMethod</h3>\n" +
                        "<pre class=\"methodSignature\">@Generated(value=\"SomeGeneratedName\",\n" +
                        "           date=\"a date\",\n" +
                        "           comments=\"some comment about the method below\")\n" +
                        "public static&nbsp;void&nbsp;annotatedMethod&#8203;(int&nbsp;i,\n" +
                        "                                   java.lang.String&nbsp;s,\n" +
                        "                                   boolean&nbsp;b)</pre>\n" +
                        "<div class=\"block\">Annotated method.</div>\n" +
                        "<dl>\n" +
                        "<dt><span class=\"paramLabel\">Parameters:</span></dt>\n" +
                        "<dd><code>i</code> - param 1</dd>\n" +
                        "<dd><code>s</code> - param 2</dd>\n" +
                        "<dd><code>b</code> - param 3</dd>\n" +
                        "</dl>\n" +
                        "</li>\n" +
                        "</ul>",
                "<ul class=\"blockList\">\n" +
                        "<li class=\"blockList\">\n" +
                        "<h3>bigGenericMethod</h3>\n" +
                        "<pre class=\"methodSignature\">public static&nbsp;&lt;T1 extends java.lang.AutoCloseable,&#8203;" +
                        "T2 extends java.lang.AutoCloseable,&#8203;T3 extends java.lang.AutoCloseable,&#8203;\n" +
                        "        T4 extends java.lang.AutoCloseable,&#8203;T5 extends java.lang.AutoCloseable," +
                        "&#8203;T6 extends java.lang.AutoCloseable,&#8203;\n" +
                        "        T7 extends java.lang.AutoCloseable,&#8203;T8 extends java.lang.AutoCloseable&gt;&nbsp;\n" +
                        "<a href=\"C.With8Types.html\" title=\"class in pkg\">C.With8Types</a>&lt;T1,&#8203;T2," +
                        "&#8203;T3,&#8203;T4,&#8203;T5,&#8203;T6,&#8203;T7,&#8203;T8&gt;&nbsp;bigGenericMethod" +
                        "&#8203;(<a href=\"C.F0.html\" title=\"interface in pkg\">C.F0</a>&lt;? extends T1&gt;&nbsp;t1,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T2&gt;&nbsp;t2,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T3&gt;&nbsp;t3,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T4&gt;&nbsp;t4,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T5&gt;&nbsp;t5,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T6&gt;&nbsp;t6,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T7&gt;&nbsp;t7,\n" +
                        "                                                       <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T8&gt;&nbsp;t8)\n" +
                        "                                                throws java.lang.IllegalArgumentException,\n" +
                        "                                                       java.lang.IllegalStateException</pre>\n" +
                        "<div class=\"block\">Generic method with eight type args.</div>\n" +
                        "<dl>\n" +
                        "<dt><span class=\"paramLabel\">Type Parameters:</span></dt>\n" +
                        "<dd><code>T1</code> - type 1</dd>\n" +
                        "<dd><code>T2</code> - type 2</dd>\n" +
                        "<dd><code>T3</code> - type 3</dd>\n" +
                        "<dd><code>T4</code> - type 4</dd>\n" +
                        "<dd><code>T5</code> - type 5</dd>\n" +
                        "<dd><code>T6</code> - type 6</dd>\n" +
                        "<dd><code>T7</code> - type 7</dd>\n" +
                        "<dd><code>T8</code> - type 8</dd>\n" +
                        "<dt><span class=\"paramLabel\">Parameters:</span></dt>\n" +
                        "<dd><code>t1</code> - param 1</dd>\n" +
                        "<dd><code>t2</code> - param 2</dd>\n" +
                        "<dd><code>t3</code> - param 3</dd>\n" +
                        "<dd><code>t4</code> - param 4</dd>\n" +
                        "<dd><code>t5</code> - param 5</dd>\n" +
                        "<dd><code>t6</code> - param 6</dd>\n" +
                        "<dd><code>t7</code> - param 7</dd>\n" +
                        "<dd><code>t8</code> - param 8</dd>\n" +
                        "<dt><span class=\"returnLabel\">Returns:</span></dt>\n" +
                        "<dd>null</dd>\n" +
                        "<dt><span class=\"throwsLabel\">Throws:</span></dt>\n" +
                        "<dd><code>java.lang.IllegalArgumentException</code></dd>\n" +
                        "<dd><code>java.lang.IllegalStateException</code></dd>\n" +
                        "</dl>\n" +
                        "</li>\n" +
                        "</ul>",
                "<ul class=\"blockListLast\">\n" +
                        "<li class=\"blockList\">\n" +
                        "<h3>bigGenericAnnotatedMethod</h3>\n" +
                        "<pre class=\"methodSignature\">@Generated(value=\"SomeGeneratedName\",\n" +
                        "           date=\"a date\",\n" +
                        "           comments=\"some comment about the method below\")\n" +
                        "public static&nbsp;&lt;T1 extends java.lang.AutoCloseable,&#8203;T2 extends " +
                        "java.lang.AutoCloseable,&#8203;T3 extends java.lang.AutoCloseable,&#8203;\n" +
                        "        T4 extends java.lang.AutoCloseable,&#8203;T5 extends java.lang.AutoCloseable," +
                        "&#8203;T6 extends java.lang.AutoCloseable,&#8203;\n" +
                        "        T7 extends java.lang.AutoCloseable,&#8203;T8 extends java.lang.AutoCloseable&gt;&nbsp;\n" +
                        "<a href=\"C.With8Types.html\" title=\"class in pkg\">C.With8Types</a>&lt;T1,&#8203;T2," +
                        "&#8203;T3,&#8203;T4,&#8203;T5,&#8203;T6,&#8203;T7,&#8203;T8&gt;&nbsp;bigGenericAnnotatedMethod" +
                        "&#8203;(<a href=\"C.F0.html\" title=\"interface in pkg\">C.F0</a>&lt;? extends T1&gt;&nbsp;t1,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T2&gt;&nbsp;t2,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T3&gt;&nbsp;t3,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T4&gt;&nbsp;t4,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T5&gt;&nbsp;t5,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T6&gt;&nbsp;t6,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T7&gt;&nbsp;t7,\n" +
                        "                                                                <a href=\"C.F0.html\" " +
                        "title=\"interface in pkg\">C.F0</a>&lt;? extends T8&gt;&nbsp;t8)\n" +
                        "                                                         throws java.lang.IllegalArgumentException,\n" +
                        "                                                                java.lang.IllegalStateException</pre>\n" +
                        "<div class=\"block\">Generic method with eight type args and annotation.</div>\n" +
                        "<dl>\n" +
                        "<dt><span class=\"paramLabel\">Type Parameters:</span></dt>\n" +
                        "<dd><code>T1</code> - type 1</dd>\n" +
                        "<dd><code>T2</code> - type 2</dd>\n" +
                        "<dd><code>T3</code> - type 3</dd>\n" +
                        "<dd><code>T4</code> - type 4</dd>\n" +
                        "<dd><code>T5</code> - type 5</dd>\n" +
                        "<dd><code>T6</code> - type 6</dd>\n" +
                        "<dd><code>T7</code> - type 7</dd>\n" +
                        "<dd><code>T8</code> - type 8</dd>\n" +
                        "<dt><span class=\"paramLabel\">Parameters:</span></dt>\n" +
                        "<dd><code>t1</code> - param 1</dd>\n" +
                        "<dd><code>t2</code> - param 2</dd>\n" +
                        "<dd><code>t3</code> - param 3</dd>\n" +
                        "<dd><code>t4</code> - param 4</dd>\n" +
                        "<dd><code>t5</code> - param 5</dd>\n" +
                        "<dd><code>t6</code> - param 6</dd>\n" +
                        "<dd><code>t7</code> - param 7</dd>\n" +
                        "<dd><code>t8</code> - param 8</dd>\n" +
                        "<dt><span class=\"returnLabel\">Returns:</span></dt>\n" +
                        "<dd>null</dd>\n" +
                        "<dt><span class=\"throwsLabel\">Throws:</span></dt>\n" +
                        "<dd><code>java.lang.IllegalArgumentException</code></dd>\n" +
                        "<dd><code>java.lang.IllegalStateException</code></dd>\n" +
                        "</dl>\n" +
                        "</li>\n" +
                        "</ul>");
    }
}
